
#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"dSFMT.h"


#define	N1		10000		//the number of random
#define	N2		10000		//the number of trial
dsfmt_t dsfmt1;				//random number



double rand_power(double alpha){	//rand : power-law
	double x = dsfmt_genrand_close_open(&dsfmt1);
	double y = pow(x, -1.0/alpha);
	return y;
}

void PDF_normal( double *data , int data_num ,double data_min , int imax ,double dx, double *output){	//PDF
	int i ;
	
	for( i=0 ; i<data_num ; i++ ){
		output[(int)( ( data[i] - data_min )/dx) ] += 1.0/data_num;
	}
	for( i=0 ; i<imax ; i++){
		output[i] /= dx;
	}
}


void PDF_log( double *data , int data_num , double data_min , int imax , double dx,  double *output){			//PDF : log-scale
	int i ;
	double *width;
	width = (double *)calloc( imax , sizeof(double));
	
	for( i = 0 ; i< data_num ; i++){
		output[ (int)(( log10(data[i]) - log10(data_min) ) / dx)] += 1.0/data_num;
	}
	for(i = 0; i< imax ; i++){
		width[i] = data_min * ( pow(10 , (i+1)*dx ) - pow(10 , i*dx ) );
	}
	for(i = 0; i< imax ; i++){
		output[i] /= width[i];
	}
	free(width);
}

int double_comp(const void *_a, const void *_b) {	//qsort
	double a = *(double *)_a;
	double b = *(double *)_b;
	
	if (a > b)		return -1;
	else if (a < b)	return 1;
	else			return 0;
}


double make_ave(int data_num , double *y){				//average
	double average = 0.0;
	int i;
	
	for(i = 0 ;i < data_num ; i++){
		average += y[i];
	}
	average /= (double)data_num;
	
	return average;
}

double make_std(int data_num , double *y){				//std_dev
	double sum = 0.0;
	double sum2 = 0.0;
	int i;
	
	for(i = 0 ;i < data_num ; i++){
		sum  += y[i];
		sum2 += y[i] * y[i];
	}
	sum  /= (double)data_num;
	sum2 /= (double)data_num;
	
	return sqrt( sum2 - sum * sum );
}


double make_med(int data_num , double *y){				//med
	double 	median;
	int 	tmp1,tmp2;
	qsort(y,data_num,sizeof(double),double_comp);
	
	if(data_num % 2 == 1){								//data_num = odd
		tmp1 = (int) (data_num -1)/2;
		median = y[tmp1];
	}
	else{												//data_num = even
		tmp1 = (int) (data_num /2) -1;
		tmp2 = (int) (data_num /2);
		median = (y[tmp1] + y[tmp2])/2.0;
	}
	return median;
}

double make_q1(int data_num , double *y){				//q1
	double quartile1;
	int tmp1, tmp2;
	qsort(y,data_num,sizeof(double),double_comp);
	
	if( data_num % 4 == 0){
		tmp1 = (int)(3 *data_num /4) -1;
		quartile1 = y[tmp1];
	}
	else{
		tmp1 = (int)(3 * data_num/4) -1;
		tmp2 = (int)(3 * data_num/4);
		quartile1 = ( y[tmp1] + y[tmp2] ) /2.0;
	}
	return quartile1;
}

double make_q3(int data_num , double *y){				//q3
	double quartile3;
	int tmp1, tmp2;
	qsort(y,data_num,sizeof(double),double_comp);
	
	if( data_num % 4 == 0){
		tmp1 = (int)(data_num /4) -1;
		quartile3 = y[tmp1];
	}
	else{
		tmp1 = (int)(data_num/4)-1;
		tmp2 = (int)(data_num/4);
		quartile3 = ( y[tmp1] + y[tmp2] ) /2.0;
	}
	return quartile3;
}

double make_qrange(int data_num , double *y){								//q_range
	double range =  make_q3(data_num , y) - make_q1(data_num , y);
	return range;
}



int main( int argc , char *argv[]){
	
	//Seed of random number
	dsfmt_init_gen_rand(&dsfmt1,1);
	
	int i, j;
	double *rand ,*output;
	double *F;
	rand = (double *)calloc( N1 , sizeof(double) );
	output = (double *)calloc( N2 , sizeof(double) );
	
	double data_min , data_max;
	int imax;
	char *fn;
	fn = (char *)calloc( 100 ,sizeof(char));
	FILE *fp;
	sprintf( fn , "output/%s_%s.dat",argv[1],argv[2]);
	if( (fp = fopen(fn , "w")) ==NULL){
		printf("FILE Error: %s\n",argv[2]);
		exit(1);
	}
	
	double alpha=atof(argv[2]);
	
	
	
	
	for( i=0; i<N2 ; i++){	//trial
		for( j=0; j<N1; j++){	//generate the random number
			rand[j] = rand_power( alpha );
		}
		
		//Calculate the stat number
		if( strcmp( argv[1] , "ave")==0 )			output[i] = make_ave(N1 , rand);
		else if( strcmp( argv[1] , "std")==0)		output[i] = make_std(N1 , rand);
		else if( strcmp( argv[1] , "med")==0 ) 		output[i] = make_med(N1 , rand);
		else if( strcmp( argv[1] , "q_r")==0 )		output[i] = make_qrange(N1 , rand);
	}
	
	
	//PDF
	double dx = atof(argv[4]);
	qsort( output  , N2 , sizeof(double) , double_comp );
	data_max = output[0];
	data_min = output[N2-1];
	
	
	if( strcmp( argv[3] , "norm")==0 ){		//normal
		imax = (int)( ( data_max - data_min )/dx ) + 1;
		F = (double * )calloc( imax , sizeof(double) );
		PDF_normal( output , N2, data_min , imax ,dx ,F );
		for(i=0;i<imax;i++)	fprintf( fp , "%f %f\n", data_min+(i+0.5)*dx ,F[i]);
		
	}
	else{									//log
		imax = (int)( (log10(data_max ) - log10(data_min) ) / dx ) + 1;
		F = (double * )calloc( imax, sizeof(double));
		PDF_log( output , N2 , data_min , imax , dx ,F );
		for(i=0;i<imax;i++)	fprintf( fp , "%f	%f\n" , data_min* pow( 10 , (i+0.5)* dx ) , F[i] );
	}
	
	
	
	fclose(fp);
	free(rand);
	free(output);
	free(F);
	free(fn);
	
	return 0;
}